import java.util.ArrayList;
import java.util.Iterator;
import java.util.NoSuchElementException;

public class SimpleIterator implements Iterable {
    @Override
    public Iterator iterator() {
        return new Iterator() {
            private boolean visited = false;
            @Override
            public boolean hasNext() {
                return !visited;
            }

            @Override
            public Object next() {
                if (!hasNext())
                    throw new NoSuchElementException("no element!");
                return SimpleIterator.this;
            }
        };
    }
}

class TreeIterator implements Iterable{
    private Iterable left = new SimpleIterator();
    private Iterable right = new SimpleIterator();


    @Override
    public Iterator iterator() {
        Iterator lefi = left.iterator();
        Iterator rigi = right.iterator();
        return new Iterator() {
            @Override
            public boolean hasNext() {
                return lefi.hasNext() || rigi.hasNext();
            }

            @Override
            public Object next() {
                if (!hasNext())
                    throw new NoSuchElementException("no element!");
                if (!lefi.hasNext())
                    return lefi.next();
                else return rigi.next();
            }
        };
    }
}


class ListIterator implements Iterable{

    private SimpleIterator[] data = new SimpleIterator[10];

    @Override
    public Iterator iterator() {

        ArrayList<Iterator> iters = new ArrayList();
        for (SimpleIterator s : data)
            iters.add(s.iterator());

        return new Iterator() {
            @Override
            public boolean hasNext() {
                for (Iterator iterator : iters){
                    if (iterator.hasNext()) return true;
                }
                return false;
            }

            @Override
            public Object next() {
                for (Iterator iterator : iters){
                    if (iterator.hasNext()) return iterator.next();
                }
                throw new NoSuchElementException("no element");
            }
        };
    }
}
